// ---------------------------------------------------------------------------------------------------------------------
//
//      Spotify For Artists Enhanced
//      Chrome Extension
//      Version 1.0
//      Author: Pavel "Pao" Antolík
//      Check out http://beatsbypao.com/en/ for more!
//
// ---------------------------------------------------------------------------------------------------------------------

// ---------- GLOBAL VARIABLES -----------------------------------------------------------------------------------------

var tableRows = [],
tracks = [];

// ---------- COMPARATORS ----------------------------------------------------------------------------------------------

function compareByRatio( a, b ) {
  if ( a.saveRatio < b.saveRatio ){
    return 1;
  }
  if ( a.saveRatio > b.saveRatio ){
    return -1;
  }
  return 0;
}

function compareByPosition( a, b ) {
  if ( a.saveRatio < b.saveRatio ){
    return -1;
  }
  if ( a.saveRatio > b.saveRatio ){
    return 1;
  }
  return 0;
}

// ---------- MUTATION OBSERVER ----------------------------------------------------------------------------------------

// setTimeout(function(){
//     // $('section[data-testid="songs-table"]').bind("DOMSubtreeModified", reload2);
//
//     var targetNode = document.querySelector('main > div');
//
//     // Options for the observer (which mutations to observe)
//     var config = { attributes: true, childList: true, subtree: true };
//
//     // Callback function to execute when mutations are observed
//     var callback = function(mutationsList, observer) {
//         for(var mutation of mutationsList) {
//             // console.log(mutation.type);
//             // console.log(mutation.target);
//             // console.log("---");
//             // if (mutation.type == 'childList') {
//             //     console.log('A child node has been added or removed.');
//             // }
//             // else if (mutation.type == 'attributes') {
//             //     console.log('The ' + mutation.attributeName + ' attribute was modified.');
//             // }
//         }
//     };
//
//     // Create an observer instance linked to the callback function
//     var observer = new MutationObserver(callback);
//
//     // Start observing the target node for configured mutations
//     observer.observe(targetNode, config);
//
// }, 1000);

// ---------- DATA & RELOADS -------------------------------------------------------------------------------------------

var getData = function(){

    var currentUrl = window.location.href;
    if(currentUrl.indexOf("artists.spotify.com/c/artist/") > -1 && currentUrl.indexOf("/music/songs") > -1){

        tracks = [];
        tableRows = $('tbody tr').get();

        if($('tbody tr').length > 0){
            //console.log("getData - GETTING DATA");

            for(var i = 0; i < tableRows.length; i++){
                var track = {};
                track.DOMelement = $(tableRows[i]);
                track.position = $(tableRows[i]).find("td:nth-child(1)").attr("title");
                track.title = $(tableRows[i]).find("td:nth-child(2)").attr("title");
                track.totalStreams = parseInt($(tableRows[i]).find("td:nth-child(4)").attr("title").replace(",", ""));
                track.uniqueListeners = parseInt($(tableRows[i]).find("td:nth-child(5)").attr("title").replace(",", ""));
                track.saves = parseInt($(tableRows[i]).find("td:nth-child(6)").attr("title").replace(",", ""));
                track.saveRatio = track.saves / (track.uniqueListeners / 100);
                track.saveRatio = isNaN(track.saveRatio) || track.saveRatio === 0 ? "--" : track.saveRatio;
                track.listensPerPerson = track.totalStreams / track.uniqueListeners;
                track.ratioPosition = 0;
                track.bubbleColor = "#000";


                if(track.saveRatio > 20) {
                    track.bubbleColor = "#4aa600";
                }else if(track.saveRatio > 10){
                    track.bubbleColor = "#f6d600";
                }else{
                    track.bubbleColor = "#cd0000";
                }
                if(isNaN(track.saveRatio)){
                    track.bubbleColor = "#a2a2a2";
                }

                tracks.push(track);
            }

            tracks.sort(compareByRatio);
            for(var i = 0; i < tracks.length; i++){
                tracks[i].ratioPosition = i + 1;
            }
            tracks.sort(compareByPosition);

            $('.appended').remove();

            var thead = $('thead tr').get();
            $(thead[0]).append('<th class="TableHeaderCell__TableHeaderCellElement-sc-1m3fko3-0 dSGteo sc-jrIrqw jJfnWv appended" style="width: 70px;">SPL</th>');
            $(thead[0]).append('<th class="TableHeaderCell__TableHeaderCellElement-sc-1m3fko3-0 dSGteo sc-jrIrqw jJfnWv appended" style="width: 110px;">Save Ratio</th>');

            for(var i = 0; i < tracks.length; i++){

                $(tracks[i].DOMelement).append('<td title="' + (isNaN(tracks[i].listensPerPerson) ? "--" : (Math.round(tracks[i].listensPerPerson * 10) / 10)) + '" class="TableCell__TableCellElement-sc-1do596v-0 hkaCxQ sc-fHxwqH bptNRJ appended" style="width: 70px;">' + (isNaN(tracks[i].listensPerPerson) ? "--" : (Math.round(tracks[i].listensPerPerson * 10) / 10) + "") + '</td>');
                $(tracks[i].DOMelement).append('<td title="' + (isNaN(tracks[i].saveRatio) ? "--" : (Math.round(tracks[i].saveRatio * 100) / 100)) + '" class="TableCell__TableCellElement-sc-1do596v-0 hkaCxQ sc-fHxwqH bptNRJ appended" style="width: 110px;"><span style="width: 18px; height: 18px; border-radius: 50%; background: ' + tracks[i].bubbleColor + '; color: white; margin-left: 10px; display: inline-block;font-size: 10px; vertical-align: middle; text-align: center; float: right; margin-top: 1px;">' + (isNaN(tracks[i].saveRatio) ? "--" : tracks[i].ratioPosition) + '</span>' + (isNaN(tracks[i].saveRatio) ? "--" : (Math.round(tracks[i].saveRatio * 100) / 100) + "%") + '</td>');
            }
        }else{
            //console.log("getData - TIMEOUT");
            setTimeout(getData, 300);
        }

    }
}

var reload = function(){
        //console.log("-- RELOAD");
        setTimeout(bindControls, 300);
        setTimeout(getData, 500);
}

// ---------- BIND FUNCTIONS -------------------------------------------------------------------------------------------

var bindSubmenu = function(){
    var currentUrl = window.location.href;
    if(currentUrl.indexOf("artists.spotify.com/c/artist/") > -1 && (currentUrl.indexOf("/music/songs") > -1 || currentUrl.indexOf("/music/releases") > -1 || currentUrl.indexOf("/music/upcoming") > -1 || currentUrl.indexOf("/music/playlists") > -1)){

        // BIND IF THE MENU IS PRESENT

        if($('main section > ul[role="tabslist"] a').length > 0){
            //console.log("bindSubmenu - BINDING");

            $('main section > ul[role="tabslist"] a').off("click");
            $('main section > ul[role="tabslist"] a').on("click", function(e){
                //console.log($(this));
                reload();
            })
        }else{
        //console.log("bindSubmenu - TIMEOUT");
        // OR SET TIMEOUT TO CHECK A BIT LATER

            setTimeout(bindSubmenu, 300);

        }
    }else{
        console.log("bindSubmenu - NO BINDING");
    }
}

var bindMainMenu = function(){

    // BIND IF THE MENU IS PRESENT

        if($('header ul li a').length > 0){
        //console.log("bindMainMenu - BINDING");
            $('header ul li a').off("click");
            $('header ul li a').on("click", function(e){
                //console.log($(this));
                reload();
            })
        }else{
        //console.log("bindMainMenu - TIMEOUT");
        // OR SET TIMEOUT TO CHECK A BIT LATER

            setTimeout(bindMainMenu, 300);

        }

}

var bindDropdown = function(){
    var currentUrl = window.location.href;
    if(currentUrl.indexOf("artists.spotify.com/c/artist/") > -1 && (currentUrl.indexOf("/music/songs") > -1 || currentUrl.indexOf("/music/releases") > -1 || currentUrl.indexOf("/music/upcoming") > -1 || currentUrl.indexOf("/music/playlists") > -1)){

        // BIND IF THE DROPDOWN IS PRESENT

        if($('ul[data-testid="tabs-layout"] a').length > 0){
        //console.log("bindDropdown - BINDING");
            $('select[data-testid="date-range-dropdown"]').off('change');
            $('select[data-testid="date-range-dropdown"]').on('change', reload);

        }else{

        // OR SET TIMEOUT TO CHECK A BIT LATER
        //console.log("bindDropdown - TIMEOUT");
            setTimeout(bindDropdown, 300);

        }
    }else{
        //console.log("bindDropdown - NO BINDING");
    }
}

var bindControls = function(){
    //console.log("-- BIND CONTROLS");
    bindMainMenu();
    bindSubmenu();
    bindDropdown();
}

// ---------- RUN THAT SHIT --------------------------------------------------------------------------------------------

$(document).ready(function(){

    // ONLY THE FIRST PAGE LOAD

    //console.log("-- LOAD");

    setTimeout(function(){
        bindControls();
        getData();
    }, 1000);
})
